#include <Wire.h>
#include <Adafruit_Sensor.h>
#include "Adafruit_SI1145.h"
#define TCAADDR 0x70  //for I2C
//START DEFINE MOTOR DRIVER
#define RIN1 10
#define RIN2 4
#define PWMR 5
#define LIN1 7
#define LIN2 8
#define PWML 6
#define STBY 9
//END DEFINE MOTOR DRIVER

#define Limit 200// IR SEEING

//---------Pro Micro
const int pingPin = 16;//FOR PING SIGNAL
//START EYE LEDS
const int LeftEye = 14;
const int RightEye = 15;
//----------END promicro


////----------Leonardo
//const int pingPin = 10;//FOR PING SIGNAL
////START EYE LEDS
//const int LeftEye = 16;
//const int RightEye = 14;
////------------END LEONARDO
//---declare sensors

Adafruit_SI1145 uv1 = Adafruit_SI1145(); //---LEFT SENSOR
Adafruit_SI1145 uv2 = Adafruit_SI1145();//-----Center SENSOR
Adafruit_SI1145 uv3 = Adafruit_SI1145();//-----Right Sensor

//CHECK ALL OF OUR PORTS
void tcaselect(uint8_t i) {
  if (i > 7) return;

  Wire.beginTransmission(TCAADDR);
  Wire.write(1 << i);
  Wire.endTransmission();
}
//END CHECK ALL OUR PORTS

void setup() {


  Serial.begin(9600);         //Sets the data rate in bits per second (baud) for serial data transmission
  Serial.println("Calibrated Light Sensor"); Serial.println("");
  while (!Serial);

  delay(1000);
  Wire.begin();

  //-----------------START SELECTING I2C PORTS
  tcaselect(2);//PORT 2 - LEFT SENSOR
  if (! uv1.begin()) {
    Serial.println("Didn't find Left Sensor");
  }
  tcaselect(5);//PORT 5 -CENTER SENSOR
  if (!uv2.begin()) {
    Serial.println("Didn't find SENSOR 1");
  }
  tcaselect(7);//PORT 7 - RIGHT SENSOR
  if (!uv3.begin()) {
    Serial.println("Didn't find SENSOR 2");
  }
  //----------------End Selecting I2C Ports
  delay(500);
  //SET UP PINS
  pinMode(RIN1, OUTPUT);
  pinMode(RIN2, OUTPUT);
  pinMode(PWMR, OUTPUT);
  pinMode(LIN1, OUTPUT);
  pinMode(LIN2, OUTPUT);
  pinMode(PWML, OUTPUT);
  pinMode(LeftEye, OUTPUT);
  pinMode(RightEye, OUTPUT);
  pinMode(STBY, OUTPUT);
  //END DEFINE PINS
}
void loop()
{
  clearLED();
  Ping();
  ReadSensors();
  Decide();
  Serial.println(); Serial.println();
  delay(100);

}
//----------FUNCTIONS

void Forward() {
  //Motor goes forward
  digitalWrite(STBY, HIGH);
  analogWrite(PWMR, 205);
  analogWrite(PWML, 205);
  digitalWrite(RIN1, HIGH);
  digitalWrite(RIN2, LOW);
  digitalWrite(LIN1, LOW);
  digitalWrite(LIN2, HIGH);
}
void Stop() {
  digitalWrite(STBY, HIGH);
  digitalWrite(RIN1, HIGH);
  digitalWrite(RIN2, HIGH);
  digitalWrite(PWMR, HIGH);
  digitalWrite(LIN1, HIGH);
  digitalWrite(LIN2, HIGH);
  digitalWrite(PWML, HIGH);
}
void RightTurn() {
  analogWrite(PWMR, 35);
  analogWrite(PWML, 253);
  digitalWrite(STBY, HIGH);
  digitalWrite(RIN1, HIGH);
  digitalWrite(RIN2, LOW);
  digitalWrite(LIN1, LOW);
  digitalWrite(LIN2, HIGH);
}
void LeftTurn() {
  analogWrite(PWMR, 255);
  analogWrite(PWML, 45);
  digitalWrite(STBY, HIGH);
  digitalWrite(RIN1, HIGH);
  digitalWrite(RIN2, LOW);
  digitalWrite(LIN1, LOW);
  digitalWrite(LIN2, HIGH);
}
void TurnAround() {
  analogWrite(PWMR, 255);
  analogWrite(PWML, 255);
  digitalWrite(STBY, HIGH);
  digitalWrite(RIN1, HIGH);
  digitalWrite(RIN2, LOW);
  digitalWrite(LIN1, LOW);
  digitalWrite(LIN2, HIGH);
}


void Ping() {
  long duration, inches, cm;
  pinMode(pingPin, OUTPUT);
  digitalWrite(pingPin, LOW);
  delayMicroseconds(2);
  digitalWrite(pingPin, HIGH);
  delayMicroseconds(5);
  digitalWrite(pingPin, LOW);
  pinMode(pingPin, INPUT);
  duration = pulseIn(pingPin, HIGH);
  //inches = microsecondsToInches(duration);
  cm = microsecondsToCentimeters(duration);

  Serial.print(cm);
  Serial.print("cm");
  Serial.println();
  if (cm >= 10 && cm <= 25)
  {

    digitalWrite(LeftEye, HIGH);
    digitalWrite(RightEye, HIGH);
    Stop();
    Serial.println("Object detected");
    delay(10);
    Ping();

  }

  clearLED();
  Serial.println("NO OBJECT");
  delay(10);

}
long microsecondsToInches(long microseconds) {
  return microseconds / 74 / 2;
}
long microsecondsToCentimeters(long microseconds) {
  return microseconds / 29 / 2;
}
//------------------Turn on eyes for detection
void Detection() {
  digitalWrite(LeftEye, HIGH);
  digitalWrite(RightEye, HIGH);
}
//-----------------Turn off eyes when clear
void clearLED() {
  digitalWrite(LeftEye, LOW);
  digitalWrite(RightEye, LOW);
}
//Check sensors let robot decide

void Decide() {
  if ( ( (uv2.readIR()) <= Limit) & ((uv1.readIR() & uv3.readIR()) >= Limit))
  {
    Forward();
    Serial.println("FORWARD");
    delay(100);
  }
  else if (  ( uv1.readIR() <= Limit ) & (( (uv2.readIR()) & (uv3.readIR())) >= Limit) )
  { //Turn right until back on track
    LeftTurn();
    Serial.println("Left Turn");
    delay(100);
  }
  else if   ( (uv3.readIR() <= Limit) &  ((uv1.readIR() & uv2.readIR()) >= Limit) )
  { //Turn left until back on track
    Serial.println(" Turn Right");
    delay(100);
    RightTurn();
  }
  else if (( (uv1.readIR()) & (uv2.readIR()) & (uv3.readIR())) >= Limit )
  {
    Serial.println("STOP over limit");
    Stop();
    delay(100);
  }
  else if (( (uv1.readIR()) & (uv2.readIR()) & (uv3.readIR())) <= Limit )
  {
    Stop();
    Serial.println("All on Black");
  }

  Forward();
  Serial.println("NO IR Values go forward");
  delay(100);

}

void ReadSensors() {
  tcaselect(2);//left sensor chosen
  Serial.println("===================");
  Serial.print("Vis1: "); Serial.println(uv1.readVisible());
  Serial.print("Left IR Reader: "); Serial.println(uv1.readIR());//read IR
  delay(10);

  tcaselect(5);//center sensor chosen
  Serial.println("===================");
  Serial.print("Vis2: "); Serial.println(uv2.readVisible());
  Serial.print("Center IR Reader: "); Serial.println(uv2.readIR());
  delay(10);

  tcaselect(7);//right sensor
  Serial.println("===================");
  Serial.print("Vis3: "); Serial.println(uv3.readVisible());
  Serial.print("Right IR Center: "); Serial.println(uv3.readIR());
  delay(10);
  //END OF CASE READING
}


